// ucalc.cpp : Implementation file
// Designed to work with 32-bit MS Visual C++ and Borland C++ Builder.
//
// UCalc Fast Math Parser (Beta S)
// Copyright (c) 1998-2007 by Daniel Corbier.  All rights reserved.
// http://www.ucalc.com

#include <stdlib.h>
#include "uCalcCPP.h"

#define StrPtr(StringArg) reinterpret_cast<DWORD>(StringArg.c_str())
#define FuncPtr(Func)     reinterpret_cast<DWORD>(Func)
#define VarPtr(NumberVar) reinterpret_cast<DWORD>(&NumberVar)

long  ucIsInitialized = 0;
long  ucLoadLibFailed = 0;
DWORD ucExtended, ucString, ucLPCSTR, ucFixedString, ucDouble;
DWORD ucLong, ucDword, ucComplex, ucByte, ucTable, ucVoid, ucStack;
DWORD ucSortedList;
DWORD ucDefaultErrorHandler;

UCEVALUATE ucEvaluate;
UCSTRRETURN ucStrReturn;
UCALC_ uCalc_;

void ucInitialize();

DWORD uCalc(long ucCommand, DWORD TextArg = 0, DWORD MiscArg1 = 0, DWORD MiscArg2 = 0, DWORD ReturnPtr = 0, DWORD t = 0)
{
   if (ucIsInitialized == ucFalse) ucInitialize();
   return uCalc_(ucCommand, TextArg, MiscArg1, MiscArg2, ReturnPtr, t);
}

char* uCalcStr(long ucCommand, DWORD TextArg = 0, DWORD MiscArg1 = 0, DWORD MiscArg2 = 0, DWORD ReturnPtr = 0, DWORD t = 0)
{
   if (ucIsInitialized == ucFalse) ucInitialize();
   return ucStrReturn(uCalc_(ucCommand, TextArg, MiscArg1, MiscArg2, ReturnPtr, t));
}

DWORD ucAddr(long AddressIndex)
{
   return uCalc(uc_GetCodeAddr, 0, AddressIndex);
}

DWORD ucDefine(String Definition, DWORD Arg1 = 0, DWORD Arg2 = 0, DWORD Arg3 = 0, DWORD t = 0)
{
   return uCalc(uc_Define, StrPtr(Definition), Arg1, Arg2, Arg3, t);
}

DWORD ucDefineFunction(String Definition, DWORD t = 0)
{
   String ItemDef;
   ItemDef = "Func: " + Definition;
   return uCalc(uc_Define, StrPtr(ItemDef), 0, 0, 0, t);
}

DWORD ucDefineOperator(String Definition, DWORD t = 0)
{
   String ItemDef;
   ItemDef = "Op: " + Definition;
   return uCalc(uc_Define, StrPtr(ItemDef), 0, 0, 0, t);
}

DWORD ucDefineVariable(String Definition, DWORD t = 0)
{
   String ItemDef;
   ItemDef = "Var: " + Definition;
   return uCalc(uc_Define, StrPtr(ItemDef), 0, 0, 0, t);
}

DWORD ucDefineSyntax(String Definition, long Rank = 1, DWORD t = 0)
{
   return uCalc(uc_DefineSyntax, StrPtr(Definition), Rank, 0, 0, t);
}

long double ucEval(String Expr, DWORD t = 0)
{
   long double Answer;
   Answer = 0;
   uCalc(uc_Eval, StrPtr(Expr), 0, 0, VarPtr(Answer), t);
   return Answer;
}

char* ucEvalStr(String Expr, DWORD ExprType = 0, DWORD t = 0)
{
   return uCalcStr(uc_EvalStr, StrPtr(Expr), ExprType, 0, 0, t);
}

char* ucInterpret(String Expr, DWORD ExprType = 0, DWORD t = 0)
{
   return uCalcStr(uc_EvalStr, StrPtr(Expr), ExprType, 1, 0, t);
}

char* ucEvaluateStr(DWORD Item)
{
   return uCalcStr(uc_EvaluateStr, 0, Item);
}

char* ucExpand(String Expr, DWORD t = 0)
{
   return uCalcStr(uc_Expand, StrPtr(Expr), 0, 0, 0, t);
}

char* ucGetErrorMessage(DWORD ErrNum = 0, DWORD t = 0)
{
   return uCalcStr(uc_GetErrorMessage, 0, ErrNum, 0, 0, t);
}

long ucGetError(DWORD t = 0)
{
   return uCalc(uc_GetErrorNumber, 0, 0, 0, 0, t);
}
long ucError(DWORD t = 0)
{
   return uCalc(uc_GetErrorNumber, 0, 0, 0, 0, t);
}
long ucSetError(long ErrorNum, DWORD t = 0)
{
   return uCalc(uc_SetErrorNumber, 0, ErrorNum, 0, 0, t);
}

DWORD ucGetItemHandle(String SymbolName, DWORD ucProperties = 0, DWORD t = 0)
{
   return uCalc(uc_GetItemData, StrPtr(SymbolName), 0, uc_Handle, ucProperties, t);
}

DWORD ucNewThread(DWORD ParentThread = 0)
{
   return uCalc(uc_NewThread, 0, ParentThread);
}

DWORD ucParse(String Expr, DWORD ExprType = 0, DWORD t = 0)
{
   return uCalc(uc_Parse, StrPtr(Expr), ExprType, 0, 0, t);
}

DWORD ucReleaseItem(DWORD ucItem)
{
   return uCalc(uc_ReleaseItem, 0, ucItem);
}

DWORD ucSetErrMsg(long ErrIndex, String ErrMessage, DWORD t = 0)
{
   return uCalc(uc_SetErrorMessage, StrPtr(ErrMessage), ErrIndex, 0, 0, t);
}

DWORD ucSetErrorNumber(long ErrorNumber, DWORD t = 0)
{
   return uCalc(uc_SetErrorNumber, 0, ErrorNumber, 0, 0, t);
}


void ucInitialize()
{
   HINSTANCE hDLL;

   hDLL  = LoadLibrary("ucalcfmp.dll");

   if (hDLL == NULL)
   {
      ucLoadLibFailed = 1;
//      return 0;
      // Host program should check LoadLibraryFailed() before continuing
   }

   ucEvaluate  = (UCEVALUATE)GetProcAddress(hDLL, "ucEvaluateExt");
   ucStrReturn = (UCSTRRETURN)GetProcAddress(hDLL, "ucStrReturn");
   uCalc_      = (UCALC_)GetProcAddress(hDLL, "uCalc");

   ucIsInitialized = ucTrue;

   uCalc(uc_Handshake);
   ucDefaultErrorHandler = uCalc(uc_AddErrorHandler, 0, ucAddr(uc_DefaultErrorHandler));

   ucSetErrMsg(uc_Err_None,                    "No Error");
   ucSetErrMsg(uc_Err_Syntax_Error,            "Syntax Error");
   ucSetErrMsg(uc_Err_Undefined_Identifier,    "Undefined Identifier");
   ucSetErrMsg(uc_Err_Unknown_Error,           "Unknown Error");
   ucSetErrMsg(uc_Err_Unrecognized_Pattern,    "Unrecognized Pattern");
   ucSetErrMsg(uc_Err_Unrecognized_Command,    "Unrecognized Command");
   ucSetErrMsg(uc_Err_Datatype_Mismatch,       "Data Type Mismatch");
   ucSetErrMsg(uc_Err_CodeBlock_Error,         "Code Block Delimiter Error");
   ucSetErrMsg(uc_Err_Invalid_Argument_Count,  "Invalid Number of Arguments");
   ucSetErrMsg(uc_Err_Definition_Aborted,      "Definition Aborted");
   ucSetErrMsg(uc_Err_Undefined_Callback,      "Undefined Callback");
   ucSetErrMsg(uc_Err_Array_Bounds_Exceeded,   "Array Bounds Exceeded");
   ucSetErrMsg(uc_Err_Float_Denormal_Operand,  "Denormal Operand");
   ucSetErrMsg(uc_Err_Float_Divide_By_Zero,    "Division By 0");
   ucSetErrMsg(uc_Err_Float_Inexact_Result,    "Inexact Result");
   ucSetErrMsg(uc_Err_Float_Invalid_Operation, "Invalid Operation");
   ucSetErrMsg(uc_Err_Float_Overflow,          "Floating Point Overflow");
   ucSetErrMsg(uc_Err_Float_Stack_Check,       "Floating Point Stack Error");
   ucSetErrMsg(uc_Err_Float_Underflow,         "Floating Point Underflow");
   ucSetErrMsg(uc_Err_Integer_Divide_By_Zero,  "Integer Division By 0");
   ucSetErrMsg(uc_Err_Integer_Overflow,        "Integer Overflow");
   ucSetErrMsg(uc_Err_Privileged_Instruction,  "Privileged Instruction Denied");

   ucExtended = ucDefine("DataType: Extended ~~ Size: 10 ~~ TypeHandler: #", ucAddr(uc_Extended));
   ucDouble   = ucDefine("DataType: Double   ~~ Size: 8  ~~ TypeHandler: # ~~ ConvertsTo: Extended, #", ucAddr(uc_Double), ucAddr(uc_Convert_Double_Extended));
   ucLong     = ucDefine("DataType: Long     ~~ Size: 4  ~~ TypeHandler: # ~~ ConvertsTo: Extended, #", ucAddr(uc_Int32),  ucAddr(uc_Convert_Int32_Extended));
   ucDword    = ucDefine("DataType: Dword    ~~ Size: 4  ~~ TypeHandler: # ~~ ConvertsTo: Extended, #", ucAddr(uc_Int32u), ucAddr(uc_Convert_Int32u_Extended));
   ucString   = ucDefine("DataType: String   ~~ Size: 4  ~~ TypeHandler: # ~~ ConvertsTo: LPCSTR", ucAddr(uc_String));
   ucFixedString = ucDefine("DataType: FixedString ~~ TypeHandler: # ~~ ConvertsTo: String ~~ ConvertsFrom: String", ucAddr(uc_String_Fixed));
   ucByte     = ucDefine("DataType: Byte     ~~ Size: 1  ~~ TypeHandler: # ~~ ConvertsTo: Extended", ucAddr(uc_Int8u));
   ucLPCSTR   = ucDefine("DataType: LPCSTR               ~~ TypeHandler: # ~~ ConvertsTo: String", ucAddr(uc_String_LPCSTR));
   ucStack    = ucDefine("DataType: Stack    ~~ Size: 4  ~~ TypeHandler: #", ucAddr(uc_Stack));
   ucSortedList = ucDefine("DataType: SortedList ~~ Size: 4  ~~ TypeHandler: #", ucAddr(uc_Stack));
   ucTable    = ucDefine("DataType: Table    ~~ Size: 4      ~~ TypeHandler: #", ucAddr(uc_Stack));
   ucVoid     = ucDefine("DataType: Void     ~~ Size: 10 ~~ TextData: Ok ~~ TypeHandler: #", ucAddr(uc_Void));

    ucExtended = ucDefine("DataType: Extended ~~ ConvertsTo: Long, #", ucAddr(uc_Convert_Extended_Int32));
ucExtended = ucDefine("DataType: Extended ~~ ConvertsTo: Dword, #", ucAddr(uc_Convert_Extended_Int32));
ucExtended = ucDefine("DataType: Extended ~~ ConvertsTo: Double, #", ucAddr(uc_Convert_Extended_Double));
ucExtended = ucDefine("DataType: Extended ~~ ConvertsTo: Byte");

ucDword = ucDefine("DataType: Dword ~~ ConvertsTo: Long");
ucDefine("DataType: Double ~~ ConvertsTo: Long");

   ucDefine("Pattern:[\\x00\\x01\\x02]");
   ucDefine("Pattern:\\x03 ~~ \\x03 ~~ DataType: #2", ucQuotedText+ucLiteral, ucString);
ucDefine("Pattern:\\x05");
   ucDefine("Pattern: :");
ucDefine("Pattern: :=");
   ucDefine("Pattern:\\[ ~~ \\]", ucCodeBlock);
   ucDefine("Pattern:\\{ ~~ \\} ~~ DataType: #2 ~~ Address: #3", ucCodeBlock, ucStack, ucAddr(uc_Func_Populate_Stack));
   ucDefine("Pattern:[\\q] ~~ [\\q] ~~ DataType: #2", ucQuotedText+ucLiteral, ucString);
   ucDefine("Pattern:['] ~~ ['] ~~ DataType: #2", ucQuotedText+ucLiteral, ucString);
ucDefine("Pattern:\\x0D\\x0A", ucStatementSep);
   ucDefine("Pattern:;", ucStatementSep);
   ucDefine("Pattern:~Eval", ucEvalInsert);
   ucDefine("Pattern:~Expand", ucExpandInsert);
   ucDefine("Pattern:\\.", ucMemberAccess);
   ucDefine("Pattern:\\~\\~");
   ucDefine("Pattern:,", ucArgSeparator);
   ucDefine("Pattern:\\( ~~ \\)", ucCodeBlock);
   ucDefine("Pattern:[ ]+", ucWhiteSpace);
   ucDefine("Pattern:[\\|+/*^$&#\\-=%@!`\\\\<>?]+", ucReducible);
   ucDefine("Pattern:([0-9]+\\.)?[0-9]*(e[-+]?[0-9]+)? ~~ DataType: #2", ucLiteral, ucExtended);
   ucDefine("Pattern:[a-z_][0-9a-z_]* ~~ Name: AlphaNum", ucAlphaNumeric);

char TempText[500];
char TempText2[500];

//+++ simplify this
strcpy(TempText, "Op: {Name} '{' {Operand} '}' [As {type}] ::=Param: {Operand} ~~ name: {Name} ~~ DataType: {type} ~~ Properties: ");
itoa(ucPrefix + ucOperator, TempText2, 10);
strcat(TempText, TempText2);
   ucDefineSyntax(TempText);

strcpy(TempText, "Op: '{' {Operand} '}' {Name} [As {type}] ::=Param: {Operand} ~~ name: {Name} ~~ DataType: {type} ~~ Properties: ");
itoa(ucPostfix + ucOperator, TempText2, 10);
strcat(TempText, TempText2);
   ucDefineSyntax(TempText);

strcpy(TempText, "Op: '{' {LeftOp} '}' {Name} '{' {RightOp} '}' [As {type}] ::=Param: {LeftOp} ~~ Param: {RightOp} ~~ name: {Name} ~~ DataType: {type} ~~ Properties: ");
itoa(ucInfix + ucOperator, TempText2, 10);
strcat(TempText, TempText2);
   ucDefineSyntax(TempText);

   ucDefineSyntax("Op: {Def} { At {addr} | ':=' {Expr:'.+'} } ::= Op: {Def} {addr: ~~ Address: {addr}} {Expr: ~~ funcbody: {Expr}}");
   ucDefineSyntax("Op: {pValue:' *[0-9]+ '} {Def} ::= Precedence: {pValue} ~~ Op: {Def}");
   ucDefineSyntax("Op:: {Def:'.+'} ::= NativeCall ~~ Address: # ~~ Op: {Def}");

   ucDefineSyntax("Var: {VarName:'[ ]*[a-z_][0-9a-z_]*'} [As {Type}] [{ = {Value} | At {Addr:' .+'} }] ::="
                  "variable ~~ name: {VarName}"
                  "{Type: ~~ DataType: {Type}}"
                  "{Value: ~~ Value: {Value}}"
                  "{Addr: ~~ Address: {Addr}}");
   ucDefineSyntax("Var: {VarName:'[ ]*[a-z_][0-9a-z_]*'} As AnyType ::= AnyType ~~ Var: {VarName}");
   ucDefineSyntax("Var: {PassBy:'( +ByRef)|( +ByVal)|( +ByHandle)|( +ByExpr)'} ::= {PassBy} ~~ Var:");
   ucDefineSyntax("Var: {Def} ... ::= Var: {Def} ~~ VariableArgList");
   ucDefineSyntax("Var: ... ::= Var: _ ~~ VariableArgList");

strcpy(TempText, "Func: {FuncName} ([{ParamList}]) [As {type:' +[a-z]+[a-z0-9]*'}] [{ At {addr} | = {Def:'.+'} }] ::= "
                 "Params: ({ParamList}) ~~ name: {FuncName} ~~ DataType: {type}"
                 "~~ Properties: ");
itoa(ucFunction + ucPrefix, TempText2, 10);
strcat(TempText, TempText2);
strcat(TempText, "{addr: ~~ Address: {addr} ~~ StdCall} ~~ funcbody: {Def}");
   ucDefineSyntax(TempText);

   ucDefineSyntax("Func:: {Def:'.+'} ::= NativeCall ~~ Address: # ~~ Func: {Def}");

   ucDefineSyntax("Params: ([{SingleParam}]) ::= IsParam ~~ { {SingleParam: Var: {SingleParam}} }");
   ucDefineSyntax("Params: ({Variable:'[^=]+'} = {Default}) ::= Params: ({Variable} ~~ TextData: {Default})");
   ucDefineSyntax("Params: ({Param1}, {OtherParams}) ::= Params: ({Param1}) ~~ Params: ({OtherParams})");
   ucDefineSyntax("Param: {ParamDef} ~~ ::= IsParam ~~ { Var: {ParamDef} } ~~");

   ucDefine("Op:: -{x}    ~~ Precedence: 25", ucAddr(uc_Op_Negate_Extended));
   ucDefine("Op:: {x As String}+{y As String} As String ~~ Precedence: 10", ucAddr(uc_Op_StrConcat));
   ucDefine("Op:: {x As Long}+{y As Long} As Long ~~ Precedence: 10", ucAddr(uc_Op_Add_Int32));

   ucExtended = ucDefine("DataType: Extended ~~ ConvertsTo: Complex, #", ucAddr(uc_Convert_Extended_Complex));
   ucComplex  = ucDefine("DataType: Complex ~~ {Var: Real As Extended} ~~ {Var: Imag As Extended = 1} ~~ TypeHandler: #", ucAddr(uc_Complex));
   ucDefine("Pattern:#i ~~ DataType: #2", ucLiteral, ucComplex);
   ucDefine("Op:: {x As Complex}+{y As Complex} As Complex ~~ Precedence: 10", ucAddr(uc_Op_Add_Complex));
   ucDefine("Op:: {x As Complex}*{y As Complex} As Complex ~~ Precedence: 10", ucAddr(uc_Op_Multiply_Complex));

   ucDefine("Op:: {x}>{y}  As Long ~~ Precedence: 5", ucAddr(uc_Op_GreaterThan_Extended));
   ucDefine("Op:: {x}<{y}  As Long ~~ Precedence: 5", ucAddr(uc_Op_LessThan_Extended));
   ucDefine("Op:: {x}>={y} As Long ~~ Precedence: 5", ucAddr(uc_Op_GreaterOrEqual_Extended));
   ucDefine("Op:: {x}<={y} As Long ~~ Precedence: 5", ucAddr(uc_Op_LessOrEqual_Extended));
   ucDefine("Op:: {x}<>{y} As Long ~~ Precedence: 5", ucAddr(uc_Op_NotEqual_Extended));
   ucDefine("Op:: {x}=={y} As Long ~~ Precedence: 5", ucAddr(uc_Op_Equal_Extended));

   ucDefine("Op:: {x As String}>{y As String}  As Long ~~ Precedence: 5", ucAddr(uc_Op_GreaterThan_Str));
   ucDefine("Op:: {x As String}<{y As String}  As Long ~~ Precedence: 5", ucAddr(uc_Op_LessThan_Str));
   ucDefine("Op:: {x As String}>={y As String} As Long ~~ Precedence: 5", ucAddr(uc_Op_GreaterOrEqual_Str));
   ucDefine("Op:: {x As String}<={y As String} As Long ~~ Precedence: 5", ucAddr(uc_Op_LessOrEqual_Str));
   ucDefine("Op:: {x As String}<>{y As String} As Long ~~ Precedence: 5", ucAddr(uc_Op_NotEqual_Str));
   ucDefine("Op:: {x As String}=={y As String} As Long ~~ Precedence: 5", ucAddr(uc_Op_Equal_Str));

   ucDefine("Op:: {x As Long} And     {y As Long}        As Long ~~ Precedence: 4", ucAddr(uc_Op_And_Int32));
   ucDefine("Op:: {x As Long} AndAlso {ByExpr y As Long} As Long ~~ Precedence: 4", ucAddr(uc_Op_AndAlso));
   ucDefine("Op:: {x As Long} Or      {y As Long}        As Long ~~ Precedence: 3", ucAddr(uc_Op_Or_Int32));
   ucDefine("Op:: {x As Long} OrElse  {ByExpr y As Long} As Long ~~ Precedence: 3", ucAddr(uc_Op_OrElse));

   ucDefine("Op:: {x}+{y} ~~ Precedence: 10", ucAddr(uc_Op_Add_Extended));
   ucDefine("Op:: {x}-{y} ~~ Precedence: 10", ucAddr(uc_Op_Subtract_Extended));
   ucDefine("Op:: {x}*{y} ~~ Precedence: 20", ucAddr(uc_Op_Multiply_Extended));
   ucDefine("Op:: {x}/{y} ~~ Precedence: 20", ucAddr(uc_Op_Divide_Extended));
   ucDefine("Op:: {x} Mod {y} ~~ Precedence: 20", ucAddr(uc_Op_Mod_Extended));
   ucDefine("Op:: {x}^{y} ~~ Precedence: 30", ucAddr(uc_Op_PowerOf_ExtToExt));
   ucDefine("Op:: {x}^{y As Long} ~~ Precedence: 30", ucAddr(uc_Op_PowerOf_ExtToLng));

   ucDefine("Op:: {x As Long}!  ~~ Precedence: 40", ucAddr(uc_Op_Factorial));
   ucDefine("Op:: {x As Long}\\{y As Long} As Long ~~ Precedence: 20", ucAddr(uc_Op_Divide_Int32));

   ucDefine("Func:: Sin(x)", ucAddr(uc_Func_Sin));
   ucDefine("Func:: Cos(x)", ucAddr(uc_Func_Cos));
   ucDefine("Func:: Tan(x)", ucAddr(uc_Func_Tan));
   ucDefine("Func:: Atan(x)", ucAddr(uc_Func_Atan));
   ucDefine("Func:: Abs(x)", ucAddr(uc_Func_Abs_Extended));
   ucDefine("Func:: Exp(x)", ucAddr(uc_Func_Exp));
   ucDefine("Func:: Log(x)", ucAddr(uc_Func_Log));
   ucDefine("Func:: Ceil(x) As Long", ucAddr(uc_Func_Ceil));
   ucDefine("Func:: Sqr(x)", ucAddr(uc_Func_SquareRoot));

   ucDefine("Func:: BaseConvert(Number As String, Base As Long)", ucAddr(uc_Func_BaseConvert));

   ucDefine("Func:: iif(ByVal cond As Long, ByExpr TruePart As String, ByExpr FalsePart As String) As String", ucAddr(uc_Func_IIf_Str));
   ucDefine("Func:: iif(ByVal cond As Long, ByExpr TruePart, ByExpr FalsePart)", ucAddr(uc_Func_IIf));
   ucDefine("Func: Inf() = 1/0");
   ucDefine("Func: NaN() = 0/0");

   ucDefine("Func:: ucAddr(index)", ucAddr(uc_Func_ucAddr));
   ucDefine("Func:: ValueAtAddr(DataType As Dword, Address As Dword, Index As Long = 0) ~~ ValueAtAddr", ucAddr(uc_Func_ValueAtAddr));

   ucDefine("Var: NullStr As String At #", ucAddr(uc_NullString));
   ucDefine("Func:: uCalc(Command As Long, TextArg As String = NullStr, MiscArg1 As Long = 0,"
            "MiscArg2 As Long = 0, ReturnPtr As Long = 0, t As Long = 0) As Long", ucAddr(uc_Func_uCalc));

   ucDefine("Func:: uCalcStr(Command As Long, TextArg As String = NullStr, MiscArg1 As Long = 0,"
            "MiscArg2 As Long = 0, ReturnPtr As Long = 0, t As Long = 0) As String", ucAddr(uc_Func_uCalcStr));

/*   ucDefine("Func: Precedence(Item As String) = uCalc("+FloatToStr(uc_GetItemData)+", Item, 0, "+FloatToStr(uc_Precedence)+")");*/

   ucDefine("Func:: uc_Loop(ByExpr DoCondition, ByExpr Expression As AnyType, ByExpr LoopCondition)", ucAddr(uc_Func_Loop));
   ucDefine("Func:: uc_For(ByHandle Counter As AnyType, Step, Start, Finish, ByExpr Expr As AnyType) As Void", ucAddr(uc_Func_For));

   ucDefine("Func:: Asc(Character As String, Position As Long = 1) As Long", ucAddr(uc_Func_Asc));
   ucDefine("Func:: Chr(Code As Long) As String", ucAddr(uc_Func_Chr));

//   return 0;
}
