// ucalc.h : Header file
// Designed to work with 32-bit MS Visual C++ and Borland C++ Builder.
//
// UCalc Fast Math Parser (Beta S)
// Copyright (c) 1998-2007 by Daniel Corbier.  All rights reserved.
// http://www.ucalc.com

#ifndef __UCALC__
#define __UCALC__

#include <wtypes.h>
#include <string.h>

// Definition properties
const ucAlphaNumeric   = 1;
const ucArgSeparator   = 2;
const ucCodeBlock      = 3;
const ucCodeBlockMatch = 4;
const ucEvalInsert     = 5;
const ucExpandInsert   = 6;
const ucMemberAccess   = 7;
const ucQuotedText     = 8;
const ucStatementSep   = 9;
const ucWhiteSpace     = 10;

const ucFuncDefEnd = 11;
const ucCurrentThread = 12;

const ucPartialBlockStart = 13;
const ucPartialBlockEnd   = 14;

const ucFunction       = 16;
const ucLiteral        = 32;
const ucReducible      = 64;
const ucSymbol         = 128;
const ucVariable       = 256;
const ucInfix          = 512;
const ucPrefix         = 1024;
const ucPostfix        = 2048;
const ucOperator       = 4096;
const ucPassOnce       = 8192;


// const ucalc Commands
const uc_Define             = 1;
const uc_DefineSyntax       = 2;
const uc_EvalDbl            = 4;
const uc_EvalExt            = 5;
const uc_EvalStr            = 6;
const uc_EvaluateDbl        = 7;
const uc_EvaluateExt        = 8;
const uc_Evaluate           = 8;
const uc_EvaluateLng        = 9;
const uc_EvaluateStr        = 10;
const uc_GetCaseSensitivity = 11;
const uc_GetCodeAddr        = 12;
const uc_GetDefaultPrecedence = 13;
const uc_GetDefaultRank     = 14;
const uc_GetDefaultType     = 15;
const uc_GetDefaultThread   = 16;
const uc_GetDefSeparator    = 17;
const uc_GetErrorNumber     = 18;
const uc_GetParentThread    = 19;
const uc_GetSyntaxEqual     = 20;
const uc_Parse              = 21;
const uc_ReleaseItem        = 22;
const uc_SetCaseSensitivity = 23;
const uc_SetCodeAddr        = 24;
const uc_SetDefaultPrecedence = 25;
const uc_SetDefaultRank     = 26;
const uc_SetDefaultThread   = 27;
const uc_SetDefaultType     = 28;
const uc_SetDefSeparator    = 29;
const uc_SetErrorNumber     = 30;
const uc_SetParentThread    = 31;
const uc_SetSyntaxEqual     = 32;
const uc_NewThread          = 33;
const uc_PartialBlockCount  = 34;

const uc_RegExWordBoundaryChars = 35;
const uc_RegExEscapeChar = 36;

const uc_AddErrorHandler = 40;
const uc_GetExpression = 60;
const uc_GetExpressionType = 61;
const uc_GetSymbol = 62;
const uc_GetItem = 63;
const uc_GetLocation = 64;

const uc_GetPatternType = 65;
const uc_GetPatternProperties = 66;

const uc_ItemClone = 70;
const uc_DataConvert = 71;
const uc_DataCopy = 72;
const uc_DataAlloc = 72;
const uc_DataFree = 73;
const uc_DataReset = 74;

const uc_SetFPU = 75;
const uc_SetFPU_HostProg = 76;
const uc_GetFPU = 77;
const uc_GetFPU_HostProg = 78;
const uc_ToggleFPU = 79;
const uc_ToggleFPU_HostProg = 80;
const uc_ErrorClear = 85;
const uc_Handshake = 90;

const uc_SetErrorMessage = 91;
const uc_GetErrorMessage = 92;

const uc_SerialNumber = 93;
const uc_SetTurbo = 94;
const uc_Expand = 95;
const uc_GetNextItem = 96;
const uc_GetItemData = 97;
const uc_SetItemData = 98;
   const uc_Properties = 1;
   const uc_Thread = 2;
   const uc_Address = 3;
   const uc_SymbolName = 4;
   const uc_Rank = 5;
   const uc_Precedence = 6;
   const uc_OverloadHandle = 7;
   const uc_OverloadCount = 8;
   const uc_ByteSize = 9;
   const uc_ArgCount = 10;
   const uc_DataType = 12;
   const uc_Handle = 13;
   const uc_Value = 14;

// Error numbers
const uc_Err_None                    = 0;
const uc_Err_Syntax_Error            = 2;
const uc_Err_Undefined_Identifier    = 3;
const uc_Err_Unknown_Error           = 4;
const uc_Err_Unrecognized_Pattern    = 5;
const uc_Err_Unrecognized_Command    = 6;
const uc_Err_Datatype_Mismatch       = 7;
const uc_Err_Invalid_Argument_Count  = 8;
const uc_Err_Definition_Aborted      = 9;
const uc_Err_CodeBlock_Error         = 10;
const uc_Err_Undefined_Callback      = 11;

const uc_Err_Array_Bounds_Exceeded   = 20;
const uc_Err_Float_Denormal_Operand  = 21;
const uc_Err_Float_Divide_By_Zero    = 22;
const uc_Err_Float_Inexact_Result    = 23;
const uc_Err_Float_Invalid_Operation = 24;
const uc_Err_Float_Overflow          = 25;
const uc_Err_Float_Stack_Check       = 26;
const uc_Err_Float_Underflow         = 27;
const uc_Err_Integer_Divide_By_Zero  = 28;
const uc_Err_Integer_Overflow        = 29;
const uc_Err_Privileged_Instruction  = 30;

// FPU Control Word Settings
const uc_FPU_Mask_InvalidOp     = 1;
const uc_FPU_Mask_DenormalOp    = 2;
const uc_FPU_Mask_ZeroDivide    = 4;
const uc_FPU_Mask_Overflow      = 8;
const uc_FPU_Mask_Underflow     = 16;
const uc_FPU_Mask_PrecisionLoss = 32;

const uc_FPU_Precision_Single   = 0;
const uc_FPU_Precision_Double   = 256;
const uc_FPU_Precision_Extended = 768;

const uc_FPU_Round_Even         = 0;
const uc_FPU_Round_Down         = 1024;
const uc_FPU_Round_Up           = 2048;
const uc_FPU_Round_Toward       = 3072;

const ucAbort = 0;
const ucResume = 1;

// Data type handler callback action modes
const uc_TypeHandler_Store = 2;
const uc_TypeHandler_StringOutput = 3;
const uc_TypeHandler_Free = 4;
const uc_TypeHandler_FuncRet = 5;
const uc_TypeHandler_Copy = 6;
const uc_TypeHandler_Alloc = 7;
const uc_TypeHandler_Reset = 8;

const uc_Callback_Undefined = 0;

// Data Type callbacks
const uc_Complex     = 1;
const uc_Currency    = 2;
const uc_Double      = 3;
const uc_Extended    = 4;
const uc_Int8        = 5;
const uc_Int8u       = 6;
const uc_Int16       = 7;
const uc_Int16u      = 8;
const uc_Int32       = 9;
const uc_Int32u      = 10;
const uc_Int64       = 11;
const uc_Int64u      = 12;
const uc_Single      = 13;
const uc_String      = 14;
const uc_String_LPCSTR= 15;
const uc_String_Fixed= 16;
const uc_Stack       = 17;
const uc_Void        = 18;

const uc_Func_StackFormat = 19;
const uc_SetVariableValue = 20;
const uc_Convert_Generic = 23;
const uc_Convert_Double_Extended = 24;
const uc_Convert_Int8_Extended = 25;
const uc_Convert_Int8u_Extended = 26;
const uc_Convert_Int16_Extended = 27;
const uc_Convert_Int16u_Extended = 28;
const uc_Convert_Int32_Extended = 29;
const uc_Convert_Int32u_Extended = 30;
const uc_Convert_Int64_Extended = 31;
const uc_Convert_Int64u_Extended = 32;
const uc_Convert_Single_Extended = 33;
const uc_Convert_Extended_Double = 37;
const uc_Convert_Extended_Single = 38;
const uc_Convert_Extended_Int16u = 39;
const uc_Convert_Extended_Int32 = 40;
const uc_Convert_Complex_Extended = 41;
const uc_Convert_Extended_Complex = 42;

// Default built-In routines
const uc_Op_Negate_Extended = 43;
const uc_Op_UnaryPlus_Extended = 44;
const uc_Op_Percentage_Extended = 45;
const uc_Op_StrConcat = 46;
const uc_Op_Add_Int32 = 47;
const uc_Op_Add_Extended = 48;
const uc_Op_Add_Complex = 49;
const uc_Op_Subtract_Int32 = 50;
const uc_Op_Subtract_Extended = 51;
const uc_Op_Multiply_Int32 = 52;
const uc_Op_Multiply_Extended = 53;
const uc_Op_Multiply_Complex = 54;
const uc_Op_Divide_Int32 = 55;
const uc_Op_Divide_Extended = 56;
const uc_Op_Mod_Int32 = 57;
const uc_Op_Mod_Extended = 58;
const uc_Op_PowerOf_ExtToExt = 59;
const uc_Op_PowerOf_ExtToLng = 60;
const uc_Op_PowerOf_CplxToCplx = 61;

const uc_Op_GreaterThan_Extended = 62;
const uc_Op_GreaterOrEqual_Extended = 63;
const uc_Op_LessThan_Extended = 64;
const uc_Op_LessOrEqual_Extended = 65;
const uc_Op_Equal_Extended = 66;
const uc_Op_NotEqual_Extended = 67;

const uc_Op_GreaterThan_Str = 68;
const uc_Op_GreaterOrEqual_Str = 69;
const uc_Op_LessThan_Str = 70;
const uc_Op_LessOrEqual_Str = 71;
const uc_Op_Equal_Str = 72;
const uc_Op_NotEqual_Str = 73;

const uc_Op_Not_Int32 = 74;
const uc_Op_And_Int32 = 75;
const uc_Op_Or_Int32 = 76;
const uc_Op_Xor_Int32 = 77;
const uc_Op_Eqv_Int32 = 78;
const uc_Op_Imp_Int32 = 79;

const uc_Op_AndAlso = 80;
const uc_Op_OrElse = 81;

const uc_Op_Factorial = 82;

const uc_Func_IIf = 83;
const uc_Func_IIf_Str = 84;

const uc_Func_Sin = 85;
const uc_Func_Cos = 86;
const uc_Func_Tan = 87;
const uc_Func_Atan = 88;
const uc_Func_Ceil = 89;
const uc_Func_Abs_Extended = 90;
const uc_Func_Abs_Int32 = 91;
const uc_Func_Exp = 92;
const uc_Func_Log = 93;
const uc_Func_Floor = 94;
const uc_Func_SquareRoot = 95;
const uc_Func_BaseConvert = 96;

const uc_Func_ExprHandle = 97;
const uc_Func_VarPtr = 98;
const uc_Func_uCalc = 99;
const uc_Func_uCalcStr = 100;
const uc_Func_ucAddr = 101;
const uc_Func_ValueAtAddr = 102;

const uc_Func_Asc = 103;
const uc_Func_Chr = 104;

const uc_Func_For  = 105;
const uc_Func_Loop = 107;

const uc_Func_LoadLibrary = 108;
const uc_Func_GetProcAddress = 109;

const uc_NullString = 110;
const uc_True = 111;

const uc_Func_PushData_Stack = 120;
const uc_Func_PopNum_Stack = 122;
const uc_Func_PopStr_Stack = 123;
const uc_Func_ReadHandle_Stack = 124;
const uc_Func_ReadNum_Stack = 125;
const uc_Func_ReadStr_Stack = 126;
const uc_Func_Count_Stack = 127;
const uc_Op_Concat_Stack = 128;
const uc_Func_Populate_Stack = 129;

const uc_Insert_Table = 130;
const uc_Delete_Table = 131;
const uc_Index_Table  = 132;
const uc_Handle_Table = 133;

// Misc other callbacks
const uc_EvaluateAddr = 150;
const uc_EvaluateTurboAddr = 151;
const uc_ExceptionHelper = 152;
const uc_DefaultErrorHandler = 153;

// Interpreter addresses
const uc_Interp_DefHandle     = 160; // Dword Ptr
const uc_Interp_ExitCommand   = 161; // String Ptr
const uc_Interp_LastResultNum = 163; // Extended Ptr
const uc_Interp_LastResultStr = 164; // String Ptr
const uc_Interp_LineContinue  = 165;
const uc_Interp_MainPrompt    = 166;
const uc_Interp_MultiLinePrompt = 167;
const uc_Interp_Prefix = 168;
const uc_Interp_Postfix = 169;
const uc_Interp_Tab = 170;
const uc_Interp_Insert = 171;

const uc_Op_Ext_Plus_Complex = 190;
const uc_Op_Ext_Times_Complex = 191;

const uc_Func_If_ElseIf = 195;
const uc_Func_SetSyntaxParams = 196;
const uc_Func_SpecialUcase = 197;

const ucTrue  = 1;
const ucFalse = 0;

const uc_First = 1;
const uc_Last  = -1;
// End Of Constants

#if defined(__BORLANDC__)
   const uc_Eval = uc_EvalExt;
#else
   const uc_Eval = uc_EvalDbl;
#endif

typedef long double (CALLBACK* UCEVALUATE)(DWORD);
typedef char*       (CALLBACK* UCSTRRETURN)(DWORD);
typedef DWORD       (CALLBACK* UCALC_)(long, DWORD, DWORD, DWORD, DWORD, DWORD);

extern UCEVALUATE ucEvaluate;
extern UCSTRRETURN ucStrReturn;
extern UCALC_ uCalc_;

extern long ucInitialized;
extern long ucLoadLibFailed;
extern DWORD ucExtended, ucString, ucLPCSTR, ucFixedString, ucDouble;
extern DWORD ucLong, ucDword, ucComplex, ucByte, ucTable, ucVoid, ucStack;
extern DWORD ucSortedList;
extern DWORD ucDefaultErrorHandler;

#endif /* __UCALC__ */

