Attribute VB_Name = "uCalcVB"
' ------------------------------------------------------------------
' uCalcVB.bas : Visual Basic include file for uCalc Fast Math Parser
' Designed to work with Visual Basic 5.0 and above.
'
' uCalc Fast Math Parser (Beta S)
' Copyright (c) 2000-2007 by Daniel Corbier.  All rights reserved.
' Visit www.ucalc.com for additional resources.
' ------------------------------------------------------------------
Option Explicit
' Definition properties
Public Const ucAlphaNumeric = 1
Public Const ucArgSeparator = 2
Public Const ucCodeBlock = 3
Public Const ucCodeBlockMatch = 4
Public Const ucEvalInsert = 5
Public Const ucExpandInsert = 6
Public Const ucMemberAccess = 7
Public Const ucQuotedText = 8
Public Const ucStatementSep = 9
Public Const ucWhiteSpace = 10

Public Const ucFuncDefEnd = 11
Public Const ucCurrentThread = 12

Public Const ucPartialBlockStart = 13
Public Const ucPartialBlockEnd = 14

Public Const ucFunction = 16
Public Const ucLiteral = 32
Public Const ucReducible = 64
Public Const ucSymbol = 128
Public Const ucVariable = 256
Public Const ucInfix = 512
Public Const ucPrefix = 1024
Public Const ucPostfix = 2048
Public Const ucOperator = 4096
Public Const ucPassOnce = 8192


' uCalc Commands
Public Const uc_Define = 1
Public Const uc_DefineSyntax = 2
Public Const uc_EvalDbl = 4
Public Const uc_EvalExt = 5
Public Const uc_EvalStr = 6
Public Const uc_EvaluateDbl = 7
Public Const uc_EvaluateExt = 8
Public Const uc_Evaluate = 8
Public Const uc_EvaluateLng = 9
Public Const uc_EvaluateStr = 10
Public Const uc_GetCaseSensitivity = 11
Public Const uc_GetCodeAddr = 12
Public Const uc_GetDefaultPrecedence = 13
Public Const uc_GetDefaultRank = 14
Public Const uc_GetDefaultType = 15
Public Const uc_GetDefaultThread = 16
Public Const uc_GetDefSeparator = 17
Public Const uc_GetErrorNumber = 18
Public Const uc_GetParentThread = 19
Public Const uc_GetSyntaxEqual = 20
Public Const uc_Parse = 21
Public Const uc_ReleaseItem = 22
Public Const uc_SetCaseSensitivity = 23
Public Const uc_SetCodeAddr = 24
Public Const uc_SetDefaultPrecedence = 25
Public Const uc_SetDefaultRank = 26
Public Const uc_SetDefaultThread = 27
Public Const uc_SetDefaultType = 28
Public Const uc_SetDefSeparator = 29
Public Const uc_SetErrorNumber = 30
Public Const uc_SetParentThread = 31
Public Const uc_SetSyntaxEqual = 32
Public Const uc_NewThread = 33
Public Const uc_PartialBlockCount = 34

Public Const uc_RegExWordBoundaryChars = 35
Public Const uc_RegExEscapeChar = 36

   Public Const uc_AddErrorHandler = 40
   Public Const uc_GetExpression = 60
   Public Const uc_GetExpressionType = 61
   Public Const uc_GetSymbol = 62
   Public Const uc_GetItem = 63
   Public Const uc_GetLocation = 64

Public Const uc_GetPatternType = 65
Public Const uc_GetPatternProperties = 66

Public Const uc_ItemClone = 70
Public Const uc_DataConvert = 71
Public Const uc_DataCopy = 72
Public Const uc_DataAlloc = 72
Public Const uc_DataFree = 73
Public Const uc_DataReset = 74

   Public Const uc_SetFPU = 75
   Public Const uc_SetFPU_HostProg = 76
   Public Const uc_GetFPU = 77
   Public Const uc_GetFPU_HostProg = 78
   Public Const uc_ToggleFPU = 79
   Public Const uc_ToggleFPU_HostProg = 80
   Public Const uc_ErrorClear = 85
   Public Const uc_Handshake = 90

   Public Const uc_SetErrorMessage = 91
   Public Const uc_GetErrorMessage = 92

   Public Const uc_SerialNumber = 93
   Public Const uc_SetTurbo = 94
   Public Const uc_Expand = 95
   Public Const uc_GetNextItem = 96
   Public Const uc_GetItemData = 97
   Public Const uc_SetItemData = 98
      Public Const uc_Properties = 1
      Public Const uc_Thread = 2
      Public Const uc_Address = 3
      Public Const uc_SymbolName = 4
      Public Const uc_Rank = 5
      Public Const uc_Precedence = 6
      Public Const uc_OverloadHandle = 7
      Public Const uc_OverloadCount = 8
      Public Const uc_ByteSize = 9
      Public Const uc_ArgCount = 10
      Public Const uc_DataType = 12
      Public Const uc_Handle = 13
      Public Const uc_Value = 14


' Error numbers
Public Const uc_Err_None = 0
Public Const uc_Err_Syntax_Error = 2
Public Const uc_Err_Undefined_Identifier = 3
Public Const uc_Err_Unknown_Error = 4
Public Const uc_Err_Unrecognized_Pattern = 5
Public Const uc_Err_Unrecognized_Command = 6
Public Const uc_Err_Datatype_Mismatch = 7
Public Const uc_Err_Invalid_Argument_Count = 8
Public Const uc_Err_Definition_Aborted = 9
Public Const uc_Err_CodeBlock_Error = 10
Public Const uc_Err_Undefined_Callback = 11

Public Const uc_Err_Array_Bounds_Exceeded = 20
Public Const uc_Err_Float_Denormal_Operand = 21
Public Const uc_Err_Float_Divide_By_Zero = 22
Public Const uc_Err_Float_Inexact_Result = 23
Public Const uc_Err_Float_Invalid_Operation = 24
Public Const uc_Err_Float_Overflow = 25
Public Const uc_Err_Float_Stack_Check = 26
Public Const uc_Err_Float_Underflow = 27
Public Const uc_Err_Integer_Divide_By_Zero = 28
Public Const uc_Err_Integer_Overflow = 29
Public Const uc_Err_Privileged_Instruction = 30

' FPU Control Word Settings
Public Const uc_FPU_Mask_InvalidOp = 1
Public Const uc_FPU_Mask_DenormalOp = 2
Public Const uc_FPU_Mask_ZeroDivide = 4
Public Const uc_FPU_Mask_Overflow = 8
Public Const uc_FPU_Mask_Underflow = 16
Public Const uc_FPU_Mask_PrecisionLoss = 32

Public Const uc_FPU_Precision_Single = 0
Public Const uc_FPU_Precision_Double = 256
Public Const uc_FPU_Precision_Extended = 768

Public Const uc_FPU_Round_Even = 0
Public Const uc_FPU_Round_Down = 1024
Public Const uc_FPU_Round_Up = 2048
Public Const uc_FPU_Round_Toward = 3072


Public Const ucAbort = 0
Public Const ucResume = 1

' Data type handler callback action modes
Public Const uc_TypeHandler_Store = 2
Public Const uc_TypeHandler_StringOutput = 3
Public Const uc_TypeHandler_Free = 4
Public Const uc_TypeHandler_FuncRet = 5
Public Const uc_TypeHandler_Copy = 6
Public Const uc_TypeHandler_Alloc = 7
Public Const uc_TypeHandler_Reset = 8

Public Const uc_Callback_Undefined = 0
' Returns error

' Data Type callbacks
Public Const uc_Complex = 1
Public Const uc_Currency = 2
Public Const uc_Double = 3
Public Const uc_Extended = 4
Public Const uc_Int8 = 5
Public Const uc_Int8u = 6
Public Const uc_Int16 = 7
Public Const uc_Int16u = 8
Public Const uc_Int32 = 9
Public Const uc_Int32u = 10
Public Const uc_Int64 = 11
Public Const uc_Int64u = 12
Public Const uc_Single = 13
Public Const uc_String = 14
Public Const uc_String_LPCSTR = 15
Public Const uc_String_Fixed = 16
Public Const uc_Stack = 17
Public Const uc_Void = 18

Public Const uc_Func_StackFormat = 19

Public Const uc_SetVariableValue = 20

Public Const uc_Convert_Generic = 23
Public Const uc_Convert_Double_Extended = 24
Public Const uc_Convert_Int8_Extended = 25
Public Const uc_Convert_Int8u_Extended = 26
Public Const uc_Convert_Int16_Extended = 27
Public Const uc_Convert_Int16u_Extended = 28
Public Const uc_Convert_Int32_Extended = 29
Public Const uc_Convert_Int32u_Extended = 30
Public Const uc_Convert_Int64_Extended = 31
Public Const uc_Convert_Int64u_Extended = 32
Public Const uc_Convert_Single_Extended = 33
Public Const uc_Convert_Extended_Double = 37
Public Const uc_Convert_Extended_Single = 38
Public Const uc_Convert_Extended_Int16u = 39
Public Const uc_Convert_Extended_Int32 = 40
Public Const uc_Convert_Complex_Extended = 41
Public Const uc_Convert_Extended_Complex = 42

' Default built-in routines.

Public Const uc_Op_Negate_Extended = 43
Public Const uc_Op_UnaryPlus_Extended = 44
Public Const uc_Op_Percentage_Extended = 45
Public Const uc_Op_StrConcat = 46
Public Const uc_Op_Add_Int32 = 47
Public Const uc_Op_Add_Extended = 48
Public Const uc_Op_Add_Complex = 49
Public Const uc_Op_Subtract_Int32 = 50
Public Const uc_Op_Subtract_Extended = 51
Public Const uc_Op_Multiply_Int32 = 52
Public Const uc_Op_Multiply_Extended = 53
Public Const uc_Op_Multiply_Complex = 54
Public Const uc_Op_Divide_Int32 = 55
Public Const uc_Op_Divide_Extended = 56
Public Const uc_Op_Mod_Int32 = 57
Public Const uc_Op_Mod_Extended = 58
Public Const uc_Op_PowerOf_ExtToExt = 59
Public Const uc_Op_PowerOf_ExtToLng = 60
Public Const uc_Op_PowerOf_CplxToCplx = 61

Public Const uc_Op_GreaterThan_Extended = 62
Public Const uc_Op_GreaterOrEqual_Extended = 63
Public Const uc_Op_LessThan_Extended = 64
Public Const uc_Op_LessOrEqual_Extended = 65
Public Const uc_Op_Equal_Extended = 66
Public Const uc_Op_NotEqual_Extended = 67

Public Const uc_Op_GreaterThan_Str = 68
Public Const uc_Op_GreaterOrEqual_Str = 69
Public Const uc_Op_LessThan_Str = 70
Public Const uc_Op_LessOrEqual_Str = 71
Public Const uc_Op_Equal_Str = 72
Public Const uc_Op_NotEqual_Str = 73

Public Const uc_Op_Not_Int32 = 74
Public Const uc_Op_And_Int32 = 75
Public Const uc_Op_Or_Int32 = 76
Public Const uc_Op_Xor_Int32 = 77
Public Const uc_Op_Eqv_Int32 = 78
Public Const uc_Op_Imp_Int32 = 79

Public Const uc_Op_AndAlso = 80
Public Const uc_Op_OrElse = 81

Public Const uc_Op_Factorial = 82

Public Const uc_Func_IIf = 83
Public Const uc_Func_IIf_Str = 84

Public Const uc_Func_Sin = 85
Public Const uc_Func_Cos = 86
Public Const uc_Func_Tan = 87
Public Const uc_Func_Atan = 88
Public Const uc_Func_Ceil = 89
Public Const uc_Func_Abs_Extended = 90
Public Const uc_Func_Abs_Int32 = 91
Public Const uc_Func_Exp = 92
Public Const uc_Func_Log = 93
Public Const uc_Func_Floor = 94
Public Const uc_Func_SquareRoot = 95
Public Const uc_Func_BaseConvert = 96

Public Const uc_Func_ExprHandle = 97
Public Const uc_Func_VarPtr = 98
Public Const uc_Func_uCalc = 99
Public Const uc_Func_uCalcStr = 100
Public Const uc_Func_ucAddr = 101
Public Const uc_Func_ValueAtAddr = 102

Public Const uc_Func_Asc = 103
Public Const uc_Func_Chr = 104

Public Const uc_Func_For = 105
Public Const uc_Func_Loop = 107

Public Const uc_Func_LoadLibrary = 108
Public Const uc_Func_GetProcAddress = 109

Public Const uc_NullString = 110
Public Const uc_True = 111

Public Const uc_Func_PushData_Stack = 120
' Public Const uc_Func_PopHandle_Stack = 121
Public Const uc_Func_PopNum_Stack = 122
Public Const uc_Func_PopStr_Stack = 123
Public Const uc_Func_ReadHandle_Stack = 124
Public Const uc_Func_ReadNum_Stack = 125
Public Const uc_Func_ReadStr_Stack = 126
Public Const uc_Func_Count_Stack = 127
Public Const uc_Op_Concat_Stack = 128
Public Const uc_Func_Populate_Stack = 129

Public Const uc_Insert_Table = 130
Public Const uc_Delete_Table = 131
Public Const uc_Index_Table = 132
Public Const uc_Handle_Table = 133

' Misc other callbacks
Public Const uc_EvaluateAddr = 150
Public Const uc_EvaluateTurboAddr = 151
Public Const uc_ExceptionHelper = 152
Public Const uc_DefaultErrorHandler = 153

' Interpreter addresses
Public Const uc_Interp_DefHandle = 160     ' DWord Ptr
Public Const uc_Interp_ExitCommand = 161   ' String Ptr
Public Const uc_Interp_LastResultNum = 163 ' Extended Ptr
Public Const uc_Interp_LastResultStr = 164 ' String Ptr
Public Const uc_Interp_LineContinue = 165
Public Const uc_Interp_MainPrompt = 166
Public Const uc_Interp_MultiLinePrompt = 167
Public Const uc_Interp_Prefix = 168
Public Const uc_Interp_Postfix = 169
Public Const uc_Interp_Tab = 170
Public Const uc_Interp_Insert = 171

Public Const uc_Op_Ext_Plus_Complex = 190
Public Const uc_Op_Ext_Times_Complex = 191

Public Const uc_Func_If_ElseIf = 195
Public Const uc_Func_SetSyntaxParams = 196
Public Const uc_Func_SpecialUCase = 197

Public Const ucTrue = 1
Public Const ucFalse = 0

Public Const uc_First = 1

   Public Const uc_Last = -1
' End Of Constants

Public ucIsInitialized As Long
Public ucExtended As Long, ucString As Long, ucLPCSTR As Long, ucFixedString As Long, ucDouble As Long
Public ucLong As Long, ucDword As Long, ucComplex As Long, ucByte As Long, ucTable As Long, ucVoid As Long, ucStack As Long
Public ucSortedList As Long
Public ucDefaultErrorHandler As Long

Public Declare Function ucEvaluate Lib "ucalcfmp.dll" Alias "ucEvaluateExt" (ByVal Item As Long) As Double
Public Declare Function ucStrReturn Lib "ucalcfmp.dll" (ByVal TextPtr As Long) As String
Public Declare Function uCalc_ Lib "ucalcfmp.dll" Alias "uCalc" (ByVal ucCommand As Long, _
       ByVal TextArg$, ByVal MiscArg1 As Long, ByVal MiscArg2 As Long, _
       ByVal ReturnPtr As Long, ByVal t As Long) As Long
       
Public Declare Function uCalc__ Lib "ucalcfmp.dll" Alias "uCalc" (ByVal ucCommand As Long, _
       ByVal TextArg As Long, ByVal MiscArg1 As Long, ByVal MiscArg2 As Long, _
       ByVal ReturnPtr As Long, ByVal t As Long) As Long


Public Function uCalc(ByVal ucCommand As Long, Optional TextArg As String = "", _
               Optional ByVal MiscArg1 As Long = 0, Optional ByVal MiscArg2 As Long = 0, _
               Optional ByVal ReturnPtr As Long = 0, Optional ByVal t As Long = 0) As Long
   If ucIsInitialized = ucFalse Then Call ucInitialize
   uCalc = uCalc_(ucCommand, TextArg, MiscArg1, MiscArg2, ReturnPtr, t)
End Function

Public Function uCalcStr(ByVal ucCommand As Long, Optional TextArg As String = "", _
               Optional ByVal MiscArg1 As Long = 0, Optional ByVal MiscArg2 As Long = 0, _
               Optional ByVal ReturnPtr As Long = 0, Optional ByVal t As Long = 0) As String
   If ucIsInitialized = ucFalse Then Call ucInitialize
   uCalcStr = ucStrReturn(uCalc_(ucCommand, TextArg, MiscArg1, MiscArg2, ReturnPtr, t))
End Function

Public Function uCalcB(ByVal ucCommand As Long, Optional ByVal TextArg As Long = 0, _
               Optional ByVal MiscArg1 As Long = 0, Optional ByVal MiscArg2 As Long = 0, _
               Optional ByVal ReturnPtr As Long = 0, Optional ByVal t As Long = 0) As Long
   If ucIsInitialized = ucFalse Then Call ucInitialize
   uCalcB = uCalc__(ucCommand, TextArg, MiscArg1, MiscArg2, ReturnPtr, t)
End Function

Public Function ucAddr(ByVal AddressIndex As Long) As Long
   ucAddr = uCalcB(uc_GetCodeAddr, 0, AddressIndex)
End Function

Public Function ucDefine(Definition As String, Optional ByVal Arg1 As Long = 0, Optional ByVal Arg2 As Long = 0, _
                  Optional ByVal Arg3 As Long = 0, Optional ByVal t As Long = 0) As Long
   ucDefine = uCalc(uc_Define, Definition, Arg1, Arg2, Arg3, t)
End Function

Public Function ucDefineFunction(Definition As String, Optional ByVal t As Long = 0) As Long
   Dim ItemDef$
   ItemDef$ = "Func: " + Definition
   ucDefineFunction = uCalc(uc_Define, ItemDef$, 0, 0, 0, t)
End Function

Public Function ucDefineOperator(Definition As String, Optional ByVal t As Long = 0) As Long
   Dim ItemDef$
   ItemDef$ = "Op: " + Definition
   ucDefineOperator = uCalc(uc_Define, ItemDef$, 0, 0, 0, t)
End Function

Public Function ucDefineVariable(Definition As String, Optional ByVal t As Long = 0) As Long
   Dim ItemDef$
   ItemDef$ = "Var: " + Definition
   ucDefineVariable = uCalc(uc_Define, ItemDef$, 0, 0, 0, t)
End Function

Public Function ucDefineSyntax(Definition As String, Optional ByVal Rank As Long = 1, Optional ByVal t As Long = 0) As Long
   ucDefineSyntax = uCalc(uc_DefineSyntax, Definition, Rank, 0, 0, t)
End Function

Public Function ucEval(Expr As String, Optional ByVal t As Long = 0) As Double
   Dim Answer As Double

   uCalc uc_EvalDbl, Expr, 0, 0, VarPtr(Answer), t
   ucEval = Answer
End Function

Public Function ucGetNextItem(Expr As String, ByRef i As Long, Optional ByVal t As Long = 0) As String
   ucGetNextItem = uCalcStr(uc_GetNextItem, Expr, 0, 0, VarPtr(i), t)
End Function

Public Function ucEvalStr(Expr As String, Optional ByVal ExprType As Long = 0, Optional ByVal t As Long = 0) As String
   ucEvalStr = uCalcStr(uc_EvalStr, Expr, ExprType, 0, 0, t)
End Function

Public Function ucInterpret(Expr As String, Optional ByVal ExprType As Long = 0, Optional ByVal t As Long = 0) As String
   ucInterpret = uCalcStr(uc_EvalStr, Expr, ExprType, 1, 0, t)
End Function

Public Function ucEvaluateStr(ByVal Item As Long) As String
   ucEvaluateStr = ucStrReturn(uCalcB(uc_EvaluateStr, 0, Item))
End Function

Public Function ucExpand(Expr As String, Optional ByVal t As Long = 0) As String
   ucExpand = uCalcStr(uc_Expand, Expr, 0, 0, 0, t)
End Function

Public Function ucGetErrorMessage(Optional ByVal ErrNum As Long = 0, Optional ByVal t As Long = 0) As String
   ucGetErrorMessage = ucStrReturn(uCalcB(uc_GetErrorMessage, 0, ErrNum, 0, 0, t))
End Function

Public Function ucGetError(Optional ByVal t As Long = 0) As Long
   ucGetError = uCalcB(uc_GetErrorNumber, 0, 0, 0, 0, t)
End Function
Public Function ucError(Optional ByVal t As Long = 0) As Long
   ucError = uCalcB(uc_GetErrorNumber, 0, 0, 0, 0, t)
End Function
Public Function ucSetError(ByVal ErrorNum As Long, Optional ByVal t As Long = 0) As Long
   ucSetError = uCalcB(uc_SetErrorNumber, 0, ErrorNum, 0, 0, t)
End Function

Public Function ucGetExpr(Optional t As Long = 0) As String
   ucGetExpr = ucStrReturn(uCalcB(uc_GetExpression, 0, 0, 0, 0, t))
End Function

Public Function ucGetSymbol(Optional t As Long = 0) As String
   ucGetSymbol = ucStrReturn(uCalcB(uc_GetSymbol, 0, 0, 0, 0, t))
End Function

Public Function ucGetItemHandle(SymbolName As String, Optional ByVal ucProperties As Long = 0, Optional ByVal t As Long = 0) As Long
   ucGetItemHandle = uCalc(uc_GetItemData, SymbolName, 0, uc_Handle, ucProperties, t)
End Function

Public Function ucNewThread(Optional ByVal ParentThread As Long = 0) As Long
   ucNewThread = uCalcB(uc_NewThread, 0, ParentThread)
End Function

Public Function ucParse(Expr As String, Optional ByVal ExprType As Long = 0, Optional ByVal t As Long = 0) As Long
   ucParse = uCalc(uc_Parse, Expr, ExprType, 0, 0, t)
End Function

Public Function ucReleaseItem(ByVal ucItem As Long) As Long
   ucReleaseItem = uCalcB(uc_ReleaseItem, 0, ucItem)
End Function

Public Function ucSetErrMsg(ByVal ErrIndex As Long, ErrMessage As String, Optional ByVal t As Long = 0) As Long
   ucSetErrMsg = uCalc(uc_SetErrorMessage, ErrMessage, ErrIndex, 0, 0, t)
End Function

Public Function ucSetErrorNumber(ByVal ErrorNumber As Long, Optional ByVal t As Long = 0) As Long
   ucSetErrorNumber = uCalcB(uc_SetErrorNumber, 0, ErrorNumber, 0, 0, t)
End Function

Sub ucInitialize()
   ucIsInitialized = ucTrue

   uCalcB uc_Handshake
   ucDefaultErrorHandler = uCalcB(uc_AddErrorHandler, 0, ucAddr(uc_DefaultErrorHandler))

   ucSetErrMsg uc_Err_None, "No Error"
   ucSetErrMsg uc_Err_Syntax_Error, "Syntax Error"
   ucSetErrMsg uc_Err_Undefined_Identifier, "Undefined Identifier"
   ucSetErrMsg uc_Err_Unknown_Error, "Unknown Error"
   ucSetErrMsg uc_Err_Unrecognized_Pattern, "Unrecognized Pattern"
   ucSetErrMsg uc_Err_Unrecognized_Command, "Unrecognized Command"
   ucSetErrMsg uc_Err_Datatype_Mismatch, "Data Type Mismatch"
   ucSetErrMsg uc_Err_CodeBlock_Error, "Code Block Delimiter Error"
   ucSetErrMsg uc_Err_Invalid_Argument_Count, "Invalid Number of Arguments"
   ucSetErrMsg uc_Err_Definition_Aborted, "Definition Aborted"
   ucSetErrMsg uc_Err_Undefined_Callback, "Undefined Callback"
   ucSetErrMsg uc_Err_Array_Bounds_Exceeded, "Array Bounds Exceeded"
   ucSetErrMsg uc_Err_Float_Denormal_Operand, "Denormal Operand"
   ucSetErrMsg uc_Err_Float_Divide_By_Zero, "Division By 0"
   ucSetErrMsg uc_Err_Float_Inexact_Result, "Inexact Result"
   ucSetErrMsg uc_Err_Float_Invalid_Operation, "Invalid Operation"
   ucSetErrMsg uc_Err_Float_Overflow, "Floating Point Overflow"
   ucSetErrMsg uc_Err_Float_Stack_Check, "Floating Point Stack Error"
   ucSetErrMsg uc_Err_Float_Underflow, "Floating Point Underflow"
   ucSetErrMsg uc_Err_Integer_Divide_By_Zero, "Integer Division By 0"
   ucSetErrMsg uc_Err_Integer_Overflow, "Integer Overflow"
   ucSetErrMsg uc_Err_Privileged_Instruction, "Privileged Instruction Denied"

   ucExtended = ucDefine("DataType: Extended ~~ Size: 10 ~~ TypeHandler: #", ucAddr(uc_Extended))
   ucDouble = ucDefine("DataType: Double   ~~ Size: 8  ~~ TypeHandler: # ~~ ConvertsTo: Extended, #", ucAddr(uc_Double), ucAddr(uc_Convert_Double_Extended))
   ucLong = ucDefine("DataType: Long     ~~ Size: 4  ~~ TypeHandler: # ~~ ConvertsTo: Extended, #", ucAddr(uc_Int32), ucAddr(uc_Convert_Int32_Extended))
   ucDword = ucDefine("DataType: Dword    ~~ Size: 4  ~~ TypeHandler: # ~~ ConvertsTo: Extended, #", ucAddr(uc_Int32u), ucAddr(uc_Convert_Int32u_Extended))
   ucString = ucDefine("DataType: String   ~~ Size: 4  ~~ TypeHandler: # ~~ ConvertsTo: LPCSTR", ucAddr(uc_String))
   ucFixedString = ucDefine("DataType: FixedString ~~ TypeHandler: # ~~ ConvertsTo: String ~~ ConvertsFrom: String", ucAddr(uc_String_Fixed))
   ucByte = ucDefine("DataType: Byte     ~~ Size: 1  ~~ TypeHandler: # ~~ ConvertsTo: Extended", ucAddr(uc_Int8u))
   ucLPCSTR = ucDefine("DataType: LPCSTR               ~~ TypeHandler: # ~~ ConvertsTo: String", ucAddr(uc_String_LPCSTR))
   ucStack = ucDefine("DataType: Stack    ~~ Size: 4  ~~ TypeHandler: #", ucAddr(uc_Stack))
   ucSortedList = ucDefine("DataType: SortedList ~~ Size: 4  ~~ TypeHandler: #", ucAddr(uc_Stack))
   ucTable = ucDefine("DataType: Table    ~~ Size: 4      ~~ TypeHandler: #", ucAddr(uc_Stack))
   ucVoid = ucDefine("DataType: Void     ~~ Size: 10 ~~ TextData: Ok ~~ TypeHandler: #", ucAddr(uc_Void))

   ucExtended = ucDefine("DataType: Extended ~~ ConvertsTo: Long, #", ucAddr(uc_Convert_Extended_Int32))
ucExtended = ucDefine("DataType: Extended ~~ ConvertsTo: Dword, #", ucAddr(uc_Convert_Extended_Int32))
ucExtended = ucDefine("DataType: Extended ~~ ConvertsTo: Double, #", ucAddr(uc_Convert_Extended_Double))
ucExtended = ucDefine("DataType: Extended ~~ ConvertsTo: Byte")

ucDword = ucDefine("DataType: Dword ~~ ConvertsTo: Long")
ucDefine ("DataType: Double ~~ ConvertsTo: Long")

   ucDefine "Pattern:[\x00\x01\x02]" ' Reserved characters
   ucDefine "Pattern:\x03 ~~ \x03 ~~ DataType: #2", ucQuotedText + ucLiteral, ucString
ucDefine "Pattern:\x05" ' Define Codeblock bootstrap
   ucDefine "Pattern: :"
ucDefine "Pattern: :="
   ucDefine "Pattern:\[ ~~ \]", ucCodeBlock
   ucDefine "Pattern:\{ ~~ \} ~~ DataType: #2 ~~ Address: #3", ucCodeBlock, ucStack, ucAddr(uc_Func_Populate_Stack)
   ucDefine "Pattern:[\q] ~~ [\q] ~~ DataType: #2", ucQuotedText + ucLiteral, ucString
   ucDefine "Pattern:['] ~~ ['] ~~ DataType: #2", ucQuotedText + ucLiteral, ucString
ucDefine "Pattern:\x0D\x0A", ucStatementSep ' Carriage Return
   ucDefine "Pattern:;", ucStatementSep
   ucDefine "Pattern:~Eval", ucEvalInsert
   ucDefine "Pattern:~Expand", ucExpandInsert
   ucDefine "Pattern:\.", ucMemberAccess
   ucDefine "Pattern:\~\~"
   ucDefine "Pattern:,", ucArgSeparator
   ucDefine "Pattern:\( ~~ \)", ucCodeBlock
   ucDefine "Pattern:[ ]+", ucWhiteSpace
   ucDefine "Pattern:[\|+/*^$&#\-=%@!`\\<>?]+", ucReducible  ' For operators
   ucDefine "Pattern:([0-9]+\.)?[0-9]*(e[-+]?[0-9]+)? ~~ DataType: #2", ucLiteral, ucExtended ' Numeric
   ucDefine "Pattern:[a-z_][0-9a-z_]* ~~ Name: AlphaNum", ucAlphaNumeric

   ucDefineSyntax "Op: {Name} '{' {Operand} '}' [As {type}] ::=" _
                 + "Param: {Operand} ~~ name: {Name} ~~ DataType: {type} ~~ Properties: " + Str$(ucPrefix + ucOperator)
   ucDefineSyntax "Op: '{' {Operand} '}' {Name} [As {type}] ::=" _
                 + "Param: {Operand} ~~ name: {Name} ~~ DataType: {type} ~~ Properties: " + Str$(ucPostfix + ucOperator)
   ucDefineSyntax "Op: '{' {LeftOp} '}' {Name} '{' {RightOp} '}' [As {type}] ::=" _
                 + "Param: {LeftOp} ~~ Param: {RightOp} ~~ name: {Name} ~~ DataType: {type} ~~ Properties: " + Str$(ucInfix + ucOperator)

   ucDefineSyntax "Op: {Def} { At {addr} | ':=' {Expr:'.+'} } ::= Op: {Def} {addr: ~~ Address: {addr}} {Expr: ~~ funcbody: {Expr}}"
   ucDefineSyntax "Op: {pValue:' *[0-9]+ '} {Def} ::= Precedence: {pValue} ~~ Op: {Def}"
   ucDefineSyntax "Op:: {Def:'.+'} ::= NativeCall ~~ Address: # ~~ Op: {Def}"

   ucDefineSyntax "Var: {VarName:'[ ]*[a-z_][0-9a-z_]*'} [As {Type}] [{ = {Value} | At {Addr:' .+'} }] ::=" _
                  + "variable ~~ name: {VarName}" _
                  + "{Type: ~~ DataType: {Type}}" _
                  + "{Value: ~~ Value: {Value}}" _
                  + "{Addr: ~~ Address: {Addr}}"
   ucDefineSyntax "Var: {VarName:'[ ]*[a-z_][0-9a-z_]*'} As AnyType ::= AnyType ~~ Var: {VarName}"
   ucDefineSyntax "Var: {PassBy:'( +ByRef)|( +ByVal)|( +ByHandle)|( +ByExpr)'} ::= {PassBy} ~~ Var:"
   ucDefineSyntax "Var: {Def} ... ::= Var: {Def} ~~ VariableArgList"
   ucDefineSyntax "Var: ... ::= Var: _ ~~ VariableArgList"


   ucDefineSyntax "Func: {FuncName} ([{ParamList}]) [As {type:' +[a-z]+[a-z0-9]*'}] [{ At {addr} | = {Def:'.+'} }] ::= " _
                  + "Params: ({ParamList}) ~~ name: {FuncName} ~~ DataType: {type}" _
                  + "~~ Properties: " + Str$(ucFunction + ucPrefix) _
                  + "{addr: ~~ Address: {addr} ~~ StdCall} ~~ funcbody: {Def}"

   ucDefineSyntax "Func:: {Def:'.+'} ::= NativeCall ~~ Address: # ~~ Func: {Def}"


   ucDefineSyntax "Params: ([{SingleParam}]) ::= IsParam ~~ { {SingleParam: Var: {SingleParam}} }"
   ucDefineSyntax "Params: ({Variable:'[^=]+'} = {Default}) ::= Params: ({Variable} ~~ TextData: {Default})"
   ucDefineSyntax "Params: ({Param1}, {OtherParams}) ::= Params: ({Param1}) ~~ Params: ({OtherParams})"
   ucDefineSyntax "Param: {ParamDef} ~~ ::= IsParam ~~ { Var: {ParamDef} } ~~"

   ucDefine "Op:: -{x}    ~~ Precedence: 25", ucAddr(uc_Op_Negate_Extended)
   ucDefine "Op:: {x As String}+{y As String} As String ~~ Precedence: 10", ucAddr(uc_Op_StrConcat)
   ucDefine "Op:: {x As Long}+{y As Long} As Long ~~ Precedence: 10", ucAddr(uc_Op_Add_Int32)

   ucExtended = ucDefine("DataType: Extended ~~ ConvertsTo: Complex, #", ucAddr(uc_Convert_Extended_Complex))
   ucComplex = ucDefine("DataType: Complex ~~ {Var: Real As Extended} ~~ {Var: Imag As Extended = 1} ~~ TypeHandler: #", ucAddr(uc_Complex))
   ucDefine "Pattern:#i ~~ DataType: #2", ucLiteral, ucComplex
   ucDefine "Op:: {x As Complex}+{y As Complex} As Complex ~~ Precedence: 10", ucAddr(uc_Op_Add_Complex)
   ucDefine "Op:: {x As Complex}*{y As Complex} As Complex ~~ Precedence: 10", ucAddr(uc_Op_Multiply_Complex)

   ucDefine "Op:: {x}>{y}  As Long ~~ Precedence: 5", ucAddr(uc_Op_GreaterThan_Extended)
   ucDefine "Op:: {x}<{y}  As Long ~~ Precedence: 5", ucAddr(uc_Op_LessThan_Extended)
   ucDefine "Op:: {x}>={y} As Long ~~ Precedence: 5", ucAddr(uc_Op_GreaterOrEqual_Extended)
   ucDefine "Op:: {x}<={y} As Long ~~ Precedence: 5", ucAddr(uc_Op_LessOrEqual_Extended)
   ucDefine "Op:: {x}<>{y} As Long ~~ Precedence: 5", ucAddr(uc_Op_NotEqual_Extended)
   ucDefine "Op:: {x}=={y} As Long ~~ Precedence: 5", ucAddr(uc_Op_Equal_Extended)

   ucDefine "Op:: {x As String}>{y As String}  As Long ~~ Precedence: 5", ucAddr(uc_Op_GreaterThan_Str)
   ucDefine "Op:: {x As String}<{y As String}  As Long ~~ Precedence: 5", ucAddr(uc_Op_LessThan_Str)
   ucDefine "Op:: {x As String}>={y As String} As Long ~~ Precedence: 5", ucAddr(uc_Op_GreaterOrEqual_Str)
   ucDefine "Op:: {x As String}<={y As String} As Long ~~ Precedence: 5", ucAddr(uc_Op_LessOrEqual_Str)
   ucDefine "Op:: {x As String}<>{y As String} As Long ~~ Precedence: 5", ucAddr(uc_Op_NotEqual_Str)
   ucDefine "Op:: {x As String}=={y As String} As Long ~~ Precedence: 5", ucAddr(uc_Op_Equal_Str)

   ucDefine "Op:: {x As Long} And     {y As Long}        As Long ~~ Precedence: 4", ucAddr(uc_Op_And_Int32)
   ucDefine "Op:: {x As Long} AndAlso {ByExpr y As Long} As Long ~~ Precedence: 4", ucAddr(uc_Op_AndAlso)
   ucDefine "Op:: {x As Long} Or      {y As Long}        As Long ~~ Precedence: 3", ucAddr(uc_Op_Or_Int32)
   ucDefine "Op:: {x As Long} OrElse  {ByExpr y As Long} As Long ~~ Precedence: 3", ucAddr(uc_Op_OrElse)

   ucDefine "Op:: {x}+{y} ~~ Precedence: 10", ucAddr(uc_Op_Add_Extended)
   ucDefine "Op:: {x}-{y} ~~ Precedence: 10", ucAddr(uc_Op_Subtract_Extended)
   ucDefine "Op:: {x}*{y} ~~ Precedence: 20", ucAddr(uc_Op_Multiply_Extended)
   ucDefine "Op:: {x}/{y} ~~ Precedence: 20", ucAddr(uc_Op_Divide_Extended)
   ucDefine "Op:: {x} Mod {y} ~~ Precedence: 20", ucAddr(uc_Op_Mod_Extended)
   ucDefine "Op:: {x}^{y} ~~ Precedence: 30", ucAddr(uc_Op_PowerOf_ExtToExt)
   ucDefine "Op:: {x}^{y As Long} ~~ Precedence: 30", ucAddr(uc_Op_PowerOf_ExtToLng)

   ucDefine "Op:: {x As Long}!  ~~ Precedence: 40", ucAddr(uc_Op_Factorial)
   ucDefine "Op:: {x As Long}\{y As Long} As Long ~~ Precedence: 20", ucAddr(uc_Op_Divide_Int32)

   ucDefine "Func:: Sin(x)", ucAddr(uc_Func_Sin)
   ucDefine "Func:: Cos(x)", ucAddr(uc_Func_Cos)
   ucDefine "Func:: Tan(x)", ucAddr(uc_Func_Tan)
   ucDefine "Func:: Atan(x)", ucAddr(uc_Func_Atan)
   ucDefine "Func:: Abs(x)", ucAddr(uc_Func_Abs_Extended)
   ucDefine "Func:: Exp(x)", ucAddr(uc_Func_Exp)
   ucDefine "Func:: Log(x)", ucAddr(uc_Func_Log)
   ucDefine "Func:: Ceil(x) As Long", ucAddr(uc_Func_Ceil)
   ucDefine "Func:: Sqr(x)", ucAddr(uc_Func_SquareRoot)

   ucDefine "Func:: BaseConvert(Number As String, Base As Long)", ucAddr(uc_Func_BaseConvert)

   ucDefine "Func:: iif(ByVal cond As Long, ByExpr TruePart As String, ByExpr FalsePart As String) As String", ucAddr(uc_Func_IIf_Str)
   ucDefine "Func:: iif(ByVal cond As Long, ByExpr TruePart, ByExpr FalsePart)", ucAddr(uc_Func_IIf)
   ucDefine "Func: Inf() = 1/0"
   ucDefine "Func: NaN() = 0/0"

   ucDefine "Func:: ucAddr(index)", ucAddr(uc_Func_ucAddr)
   ucDefine "Func:: ValueAtAddr(DataType As Dword, Address As Dword, Index As Long = 0) ~~ ValueAtAddr", ucAddr(uc_Func_ValueAtAddr)

   ucDefine "Var: NullStr As String At #", ucAddr(uc_NullString)
   ucDefine "Func:: uCalc(Command As Long, TextArg As String = NullStr, MiscArg1 As Long = 0," _
           + "MiscArg2 As Long = 0, ReturnPtr As Long = 0, t As Long = 0) As Long", ucAddr(uc_Func_uCalc)

   ucDefine "Func:: uCalcStr(Command As Long, TextArg As String = NullStr, MiscArg1 As Long = 0," _
           + "MiscArg2 As Long = 0, ReturnPtr As Long = 0, t As Long = 0) As String", ucAddr(uc_Func_uCalcStr)

   ucDefine "Func: Precedence(Item As String) = uCalc(" + Str$(uc_GetItemData) + ", Item, 0, " + Str$(uc_Precedence) + ")"

ucDefine "Func:: uc_Loop(ByExpr DoCondition, ByExpr Expression As AnyType, ByExpr LoopCondition)", ucAddr(uc_Func_Loop)
ucDefine "Func:: uc_For(ByHandle Counter As AnyType, Step, Start, Finish, ByExpr Expr As AnyType) As Void", ucAddr(uc_Func_For)

ucDefine "Func:: Asc(Character As String, Position As Long = 1) As Long", ucAddr(uc_Func_Asc)
ucDefine "Func:: Chr(Code As Long) As String", ucAddr(uc_Func_Chr)

End Sub
